/*
    Bitio: Bit level input and output

    05/01/97 JSP Creation
*/

#include <stdio.h>

#include "bitio.h"

BitFile::BitFile (FILE *fp)
{
    _fp = fp;
    _bit_reg_width = 0;
}

void BitFile::Write (unsigned long bits, unsigned width)
{
    while (width)
    {
        // If the register is full, write it

        if (_bit_reg_width == 8)
        {
            fputc (_bit_reg, _fp);
            _bit_reg_width = 0;
        }

        // Shift leftmost bit of bits into register

        _bit_reg <<= 1;
        _bit_reg |= ((bits >> (width - 1)) & 1);
        ++_bit_reg_width;
        --width;
    }
}

char BitFile::Read (unsigned width)
{
    if (width > 8)
        throw "The specified width is too big";

    char ret = 0;

    // Mask off unused bits

    _bit_reg &= (1 << _bit_reg_width) - 1;

    // Load ret with return value

    if (width <= _bit_reg_width)
    {
        ret = _bit_reg >> (_bit_reg_width - width);
        _bit_reg_width -= width;
    }
    else
    {
        ret = _bit_reg;
        width -= _bit_reg_width;
        _bit_reg_width = 8;
        _bit_reg = fgetc (_fp);
        ret <<= width;
        ret |= (_bit_reg >> (8 - width));
        _bit_reg_width -= width;
    }

    return ret;
}

void BitFile::Flush ()
{
    if (_bit_reg_width)
    {
        while (_bit_reg_width < 8)
        {
            _bit_reg <<= 1;
            ++_bit_reg_width;
        }

        fputc (_bit_reg, _fp);
    }
}