/*
    LPCodec: Open a TGA file, convert it to a LP file,
    then read it back in again and write it as a TGA 
    file.

    The user can specify the quantization parameters
    on the command line in order to get a feel for
    the compression/image quality tradeoff.

    Intermediate levels of the pyramid can be output
    by changing the SAVE_INTERMEDIATES flag at 
    compile time.

    CRTDBG code is Microsoft v4.2 specific.

    05/01/97 JSP Creation
*/

#include <crtdbg.h>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>

// MS doesn't define this struct in the ndebug block
// of crtdbg.h

#ifdef NDEBUG
#pragma warning (disable : 4101)
struct _CrtMemState {};
#endif

#include "lp.h"
#include "tga.h"

enum { LEVELS = 5 };
enum { SAVE_INTERMEDIATES = 1};

main (int argc, char **argv)
{
    try
    {
        _CrtMemState ms1;
        _CrtMemCheckpoint (&ms1);

        { // MemCheck Block

        if (argc < 2)
            throw "usage: test1 <tga filename> [bpp0 [bpp1 [bpp2 [bpp3 [bpp4]]]]]";

        cout << argv[1] << ":" << endl;

        TGA tga;

        tga.Load (argv[1]);

        cout << tga.Width () << " X " << tga.Height ()
            << " greyscale image" << endl;

        unsigned bpp[LEVELS] = {3, 4, 6, 7, 8};

        for (int arg = 2; arg < argc; arg++)
            bpp[arg - 2] = atoi (argv[arg]);

        cout << "Pyramid levels: " << (int) LEVELS << endl;
        cout << "Quantization levels: ";

        for (int index = 0; index < 5; index++)
            cout << bpp[index] << " ";

        cout << endl;

        LP lp (tga.Width (), 
            tga.Height (), 
            (unsigned char *) tga.Image (), 
            LEVELS,
            bpp);

        lp.Compress ();

        unsigned width;
        unsigned height;

        if (SAVE_INTERMEDIATES)
        {
            for (unsigned level = 0; level < LEVELS; level++)
            {
                lp.GetDimensions (level, width, height);

                TGA save_tga;

                save_tga.Load (width, 
                    height, 
                    8, // bpp
                    TGA_GREY_SCALE,
                    lp.GetRImage (level));

                char filename[256];

                sprintf (filename, "r%d.tga", level);

                cout << "Saving " << filename << endl;

                save_tga.Save (filename);

                if (level != LEVELS - 1)
                {
                    save_tga.Load (width, 
                        height, 
                        8, // bpp
                        TGA_GREY_SCALE,
                        lp.GetLImage (level));

                    char filename[256];

                    sprintf (filename, "l%d.tga", level);

                    cout << "Saving " << filename << endl;

                    save_tga.Save (filename);
                }
            }
        }

        cout << "Total Entropy = " << 
            lp.Entropy () << " bpp" << endl;

        lp.Quantize ();

        cout << "Quantized Entropy = " << 
            lp.Entropy (1) << 
            " bpp" << endl;

        char *filename = "out.lp";

        cout << "Saving " << filename << endl;

        lp.Save (filename);

        cout << "Reading " << filename << endl;

        LP restored_lp (filename);

        restored_lp.Unquantize ();
        restored_lp.Decompress ();
        
        TGA save_tga;

        width = tga.Width ();
        height = tga.Height ();

        save_tga.Load (width, 
            height, 
            8, // bpp
            TGA_GREY_SCALE,
            restored_lp.GetRImage (0));

        filename = "out.tga";

        cout << "Saving " << filename << endl;

        save_tga.Save (filename);

        if (!_CrtCheckMemory ())
            throw "The heap is corrupt";

        } // MemCheck Block

        _CrtMemState ms2;
        _CrtMemCheckpoint (&ms2);
        _CrtMemState diff;

        if (_CrtMemDifference (&diff, &ms1, &ms2))
        {
            _CrtDumpMemoryLeaks ();
            throw "Memory leaks detected";
        }

        return 0;
    }
    catch (const char *e)
    {
        cerr << e << endl;

        return -1;
    }
}
